/**
 * Layout configuration widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}                         ZhPreferences                         Service to handle preferences persistance
 * @param {ZhLayoutPreferences}                   ZhLayoutPreferences                   Service to handle layout preferences methods
 * @param {ZhLayoutWidgetsBuilder}                ZhLayoutWidgetsBuilder                Service to create widgets to handle layouts
 * @param {ScreenService}                         ScreenService                         Service to manipulate screen
 * @param {MessageService}                        MessageService                        Service to show messages
 * @param {ZHPromise}                             ZHPromise                             Service to handle promises
 * @param {$timeout}                              $timeout                              Service to handle timeouts
 * @param {ZhLayoutCreateLayoutOptionsController} ZhLayoutCreateLayoutOptionsController Controller to manipulate create layout options widget
 * @param {ZhLayoutImportLayoutController}        ZhLayoutImportLayoutController        Controller to manipulate import layout widget
 */
function ZhLayoutConfigController(ZhPreferences, ZhLayoutPreferences, ZhLayoutWidgetsBuilder, ScreenService, MessageService, ZHPromise, $timeout, ZhLayoutCreateLayoutOptionsController, ZhLayoutImportLayoutController){

    var configWidgetPromise = null;

    /**
     * Retrieve widget that handles layouts configuration
     *
     * @since 3.0.0
     *
     * @param  {Widget}     listWidget Widget that is being used to create layouts for
     * @param  {DataSource} dataSource DataSource to use on config widget
     *
     * @return {Promise<Widget>}       Widget that handles layouts configuration
     */
    this.getConfigWidget = function(listWidget, dataSource){
        if(configWidgetPromise === null){
            configWidgetPromise = ZhLayoutWidgetsBuilder.factoryLayoutConfigurationWidget();
        }

        var userID = ZhPreferences.getUserID();

        return ZHPromise.all([userID, configWidgetPromise]).then(function(args){
            var configWidget = args[1];

            configWidget.listWidget = listWidget;
            configWidget.dataSourceFilter = [
                {'name': 'WIDGET_ID', 'operator': '=', 'value': listWidget.id}
            ];
            dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == (listWidget.appliedLayout ? listWidget.appliedLayout.ID : 'ORIGINAL');
            });
            configWidget.dataSource = dataSource;

            return configWidget;
        });
    };

    /**
     * Open widget to create a new layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layout configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.openCreateLayoutWidget = function(configWidget){
        var listWidget = configWidget.listWidget;
        var pdfReportOrientation = listWidget.reportOrientation || "PORTRAIT";

        return ZhPreferences.getUserID().then(function(userID){
            return {
                '__is_new': true,
                'ID': null,
                'WIDGET_ID': listWidget.id,
                'USER_ID': userID,
                'LAYOUT_CHANGES': JSON.stringify(createDiffDataStructure(listWidget.container, listWidget)),
                'PDF_REPORT_ORIENTATION': pdfReportOrientation,
                'LAST_MODIFIED_DATE': null
            };
        }).then(function(preference){
            return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(createLayoutOptionsWidget){
                createLayoutOptionsWidget.createNewLayoutWidget = {'configWidget': configWidget};

                $timeout(function(){
                    ScreenService.openPopup(createLayoutOptionsWidget);
                });
            });
        });

    };

    function createDiffDataStructure(container, widget){
        var containerDiff = {};
        createWidgetsDataStructure(widget.id, container.widgets, containerDiff);
        return containerDiff;
    }

    function createWidgetsDataStructure(widgetID, widgets, containerDiff){
        containerDiff.widgets = [];
        createWidgetsObjects(containerDiff.widgets, widgets, widgetID);
        if(Util.isArray(widgets) && widgets.length > 0){
            widgets.forEach(function(eachWidget){
                createWidgetsDataStructure(widgetID, eachWidget.widgets, getWidgetObjectByID(containerDiff.widgets, eachWidget.id));
            });
        }
    }

    function createWidgetsObjects(widgetsDiff, widgetsArray, widgetID){
        if(Util.isArray(widgetsArray) && widgetsArray.length > 0){
            widgetsArray.forEach(function(eachWidget){
                var properties = {
                    "id": eachWidget.id
                };
                if(eachWidget.id == widgetID){
                    properties.fields = [];

                }
                widgetsDiff.push(properties);
            });
        }
    }

    function getWidgetObjectByID(widgets, widgetId){
        return widgets.filter(function(eachWidgetObject){
            return eachWidgetObject.id == widgetId;
        }).shift();
    }

    /**
     * Open popup to edit a saved layout
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout   Layout to edit
     *
     * @return {Promise<Widget>} Widget used to edit the layout
     */
    this.editLayout = function(configWidget, layout){
        return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(layout, configWidget).then(function(createLayoutOptionsWidget){
            createLayoutOptionsWidget.createNewLayoutWidget = {'configWidget': configWidget};

            $timeout(function(){
                ScreenService.openPopup(createLayoutOptionsWidget);
            });
        });
    };

    /**
     * Delete layouts
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that contains the layouts to delete
     *
     * @return {Promise}             Promise resolved after layouts are deleted
     */
    this.deleteLayouts = function(configWidget){
        var layoutsToDelete = configWidget.getCheckedRows();

        var defaultLayout = layoutsToDelete.filter(function(layout){
            return layout.DEFAULT_LAYOUT;
        });
        if(defaultLayout.length){
            MessageService.showMessageByCode('zh-layout-preferences#delete_error_default');

            return;
        }

        return MessageService.confirmMessageByCode('zh-layout-preferences#delete', {
            'NR_OF_LAYOUTS': layoutsToDelete.length
        }).then(function(){
            return ZhLayoutPreferences.deleteLayouts(layoutsToDelete).then(function(){
                configWidget.dataSource.clearCheckedRows();
                return ZhLayoutPreferences.updateLayouts(configWidget.listWidget.id, configWidget.dataSource);
            }).catch(function(error){
                return MessageService.getMessageByCode('zh-layout-preferences#'+error.message).then(function(errorMessage) {
                    return MessageService.showMessageByCode('zh-layout-preferences#delete_error', {
                        'ERROR': ScreenService.i18n(errorMessage)
                    });
                });
            });
        });
    };

    /**
     * Set a layout as the default preference
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layouts configuration
     * @param  {Layout} layout       Layout that it's default value has changed
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateDefaultLayout = function(configWidget, layout){
        return ZhPreferences.changeDefaultPreferences(configWidget, 'DEFAULT_LAYOUT', layout);
    };

    /**
     * Apply Layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layouts configuration
     * @param  {Layout} layout       Layout to be applied
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateAppliedLayout = function(configWidget, layout){
        var listWidget = configWidget.listWidget;

        return ZhLayoutPreferences.applyLayout(listWidget, layout).then(function(){
            configWidget.dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == layout.ID;
            });
        });
    };

    /**
     * Download preference as file
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout       Layout to export
     *
     * @return {Promise<Layout>}     Layout exportado
     */
    this.downloadPreferenceFile = function(layout){
        return ZhLayoutPreferences.downloadPreferenceFile(layout);
    };

    /**
     * Open widget to import a new layout from file
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layout configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.importPreferenceFile = function(configWidget){
        return ZhLayoutImportLayoutController.getImportLayoutWidget().then(function(importLayoutWidget){
            importLayoutWidget.newRow();
            importLayoutWidget.configWidget = configWidget;
            return ScreenService.openPopup(importLayoutWidget);
        });
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutConfigController', ZhLayoutConfigController);
});